/********************************
 *   Graphing Demo              *
 *   Author: lobster            *
 *   Last Modified: 09/03/05    *
 ********************************/

#include <nds.h>
#include <math.h>

int DrawGLScene();

GLfloat ambientLight[] = {0.8, 0.8, 0.5};
GLfloat diffuseLight[] = {0.25, 0.25, 0.25};
GLfloat lightPosition[] = {4, 0, 0, 0};
float radius = -10;
float phi=0, theta=0;
float xmin=-2, xmax=2, xstep=0.2, ymin=-2, ymax=2, ystep=0.2;


int main()
{
	powerON(POWER_ALL);
	videoSetMode(MODE_0_3D);
	irqInitHandler(irqDefaultHandler);
	irqSet(IRQ_VBLANK, 0);
	glViewPort(0,0,255,191);
	glClearColor(0,0,0);
	glClearDepth(0x7FFF);
	
	while(1)
	{
		scanKeys();
		
		if ((keysHeld() & KEY_R))
			radius += 0.5;
		if ((keysHeld() & KEY_L))
			radius -= 0.5;
		if ((keysHeld() & KEY_LEFT))
			theta += 3;
		if ((keysHeld() & KEY_RIGHT))
			theta -= 3;
		if ((keysHeld() & KEY_UP))
			phi -= 3;
		if ((keysHeld() & KEY_DOWN))
			phi += 3;
		if ((keysDown() & KEY_START))
		{
			theta = 0;
			phi = 0;
		}
		
		glReset();
		gluPerspective(35, 256 / 192, 0.1, 100);
		glMatrixMode(GL_MODELVIEW);
		
		glLight(0, RGB15(31,31,31), 0, 0, floatov10(-1.0));
		glMaterialf(GL_AMBIENT, RGB15(16,16,0));
		glMaterialf(GL_DIFFUSE, RGB15(16,16,0));
		glMaterialf(GL_SPECULAR, BIT(15) | RGB15(31,31,0));
		
		glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE);
		
		glPushMatrix();
		DrawGLScene();
		glPopMatrix(1);
		glFlush();
	}
	
	return 0;
}

int DrawGLScene()
{
	gluLookAt(0,0,0,-100,0,0,0,0,1);
	glTranslatef(radius,0,0);
	glRotatef(theta,0,0,1);
	glRotatef(phi,1,0,0);
	glPushMatrix();
	for(float y = ymin; y < ymax; y += ystep)
	{
		glBegin(GL_TRIANGLE_STRIP);
			glNormal3f(0,0,1);
			for(float x = xmin; x < xmax; x += xstep)
			{
				if(x*y < 0)
					glColor3f(-x*y,-x*y,-x*y);
				else
					glColor3f(x*y,x*y,x*y);
				glVertex3f(x, y, x*y);
				if(x*(y+ystep) < 0)
					glColor3f(-x*(y+ystep),-x*(y+ystep),-x*(y+ystep));
				else
					glColor3f(x*(y+ystep),x*(y+ystep),x*(y+ystep));
				glVertex3f(x, y+ystep, x*(y+ystep));
				if((x+xstep) < 0)
					glColor3f(-(x+xstep)*y,-(x+xstep)*y,-(x+xstep)*y);
				else
					glColor3f((x+xstep)*y,(x+xstep)*y,(x+xstep)*y);
				glVertex3f(x+xstep, y, (x+xstep)*y);
				if((x+xstep)*(y+ystep) < 0)
					glColor3f(-(x+xstep)*(y+ystep),-(x+xstep)*(y+ystep),-(x+xstep)*(y+ystep));
				else
					glColor3f((x+xstep)*(y+ystep),(x+xstep)*(y+ystep),(x+xstep)*(y+ystep));
				glVertex3f(x+xstep, y+ystep, (x+xstep)*(y+ystep));
			}
		glEnd();
	}
	glPopMatrix(1);
	
	return TRUE;
}
